#!/usr/bin/env python 3
# -*- coding: utf-8 -*-

#
# Copyright (c) 2020 PanXu, Inc. All Rights Reserved
#
"""
main tag 管理器

Authors: PanXu
Date:    2020/10/16 16:25:00
"""


class MainTagManager:
    """
    所有的 main tag 从这里获取
    """

    EPOCH_USED_TIME = "epoch_used_time"
    EPOCH_LOSS = "epoch_loss"
    EPOCH_METRIC = "epoch_metric"
