#!/usr/bin/env python 3
# -*- coding: utf-8 -*-

#
# Copyright (c) 2020 Baidu.com, Inc. All Rights Reserved
#
"""
token

Authors: panxu(panxu@baidu.com)
Date:    2020/05/13 15:31:00
"""

class Token:

    def __init__(self, text: str):
        self.text = text