#!/usr/bin/env python 3
# -*- coding: utf-8 -*-

#
# Copyright (c) 2020 PanXu, Inc. All Rights Reserved
#
"""
component 内建的 key

Authors: PanXu
Date:    2020/10/27 19:57:00
"""


class ComponentBuiltinKey:
    """
    组件内建 key 会用在config的文件中
    """
    OBJECT = "__object__"
    TYPENAME = "__type__"
    NAME_SPACE = "__name_space__"
    EASYTEXT_NAME_SPACE = "__easytext__"
    IS_TRAINING = "is_training"

