#!/usr/bin/env python 3
# -*- coding: utf-8 -*-

#
# Copyright (c) 2020 PanXu, Inc. All Rights Reserved
#
"""
工厂初始化

Authors: PanXu
Date:    2020/10/27 14:53:00
"""

from .component_factory import ComponentFactory
