#!/usr/bin/env python 3
# -*- coding: utf-8 -*-

#
# Copyright (c) 2020 PanXu, Inc. All Rights Reserved
#
"""
初始化文件

Authors: PanXu
Date:    2020/10/20 17:24:00
"""

from .rnn_seq2seq import RnnSeq2Seq
